use itertools::join;

use crate::{
    echo_to_term_frame, lua_state::LuaState, parsing::parse_commands, GlobalCell, TermFrame,
};

fn reentrant_command_handler(
    lua_state: &mut LuaState,
    globals: &GlobalCell,
    term_frame: &TermFrame,
    command_in: &str,
) {
    echo_to_term_frame(globals, term_frame, "\r").unwrap_or(());
    lua_state.set_current_frame(term_frame);
    for command in parse_commands(command_in).commands {
        match command.split_out_command() {
            None => (),
            Some((cmd, rest)) => {
                if cmd == "##" {
                    match lua_state.execute(&join(rest.arguments.iter(), " ")) {
                        Ok(()) => (),
                        Err(msg) => {
                            echo_to_term_frame(globals, term_frame, &format!("{}\r\n", msg))
                                .unwrap_or(())
                        }
                    }
                }
            }
        }
    }
}

pub fn command_handler(globals: &GlobalCell, term_frame: &TermFrame, command_in: &str) {
    match globals.lua_engine.try_borrow_mut() {
        Err(_) => echo_to_term_frame(
            globals,
            term_frame,
            "Attempt to re-enter command handler during processing.\r\n",
        )
        .unwrap_or(()), // Ignore error handling error.
        Ok(mut lua_state_m) => {
            reentrant_command_handler(&mut lua_state_m, globals, term_frame, command_in)
        }
    }
}
